import signal
import sys
import threading

from libraries.base.config import ConfigReader
from modules.controller.controller import AuraController
from modules.communication.redis.adapter import ServerRedisAdapter
from modules.web.routes import Routes


class Aura():
    server = None
    config = None
    messenger = None
    controller = None

    # ------------------------------------------------------------------------------------------ #
    def __init__(self):
        self.config = ConfigReader()
        self.config.loadConfig()

        server = object

        self.controller = AuraController(self.config)

        self.messenger = ServerRedisAdapter()
        self.messenger.set_controller(self.controller)
        self.messenger.set_config(self.config)

        def receive_signal(signum, stack):
            print("received signal")
            server.reload()

        signal.signal(signal.SIGUSR1, receive_signal)

    def join_comm(self):
        # start listener thread
        self.messenger.start()

    def start_web_service(self):
        r = Routes()
        #r.app.run()


# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    aura = Aura()
    aura.join_comm()
    aura.start_web_service()
# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()