__author__ = 'gg'
# -*- coding: utf-8 -*-

import datetime, os, urllib, sys
import time
import decimal

from sqlalchemy import orm, Boolean, Column, Date, DateTime, Float, Integer, String, Text, Time, ForeignKey, ForeignKeyConstraint
from sqlalchemy.orm import relationship
from sqlalchemy.sql.expression import false
from libraries.database.database import db


class AuraDatabaseModel:
    def store(self, add=False, commit=False):
        #obj_session = db.session.object_session(self)
        #if obj_session is not None:
        #    obj_session.add(self)
        #    if commit:
        #        obj_session.commit()

        if add:
            db.session.add(self)
        if commit:
            db.session.commit()

    def delete(self, commit=False):
        #obj_session = db.session.object_session(self)

        #if obj_session is not None:
        #    obj_session.delete(self)
        #    if commit:
        #        obj_session.commit()

        db.session.delete(self)
        if commit:
            db.session.commit()

    def _asdict(self):
        return self.__dict__

    @staticmethod
    def recreate_db(systemexit = False):
        manualschedule = Schedule()
        manualschedule.schedule_id = 0
        manualschedule.show_name = "Manual Show"

#        fallback_trackservice_schedule = TrackServiceSchedule()
#        fallback_trackservice_schedule.ts_schedule_id = 0

        print("Recreating Database...")
        db.drop_all()
        print("all dropped. creating...")
        db.create_all()
        print("inserting manual scheduling possibility and fallback trackservice schedule")
        db.session.add(manualschedule)
#        db.session.add(fallback_trackservice_schedule)
        print("all created. commiting...")
        db.session.commit()
        print("Database recreated!")

        if systemexit:
            sys.exit(0)


# ------------------------------------------------------------------------------------------ #
class Schedule(db.Model, AuraDatabaseModel):
    """
    One specific Schedule for a show on a timeslot
    """
    __tablename__ = 'schedule'

    # primary and foreign keys
    schedule_id = Column(Integer, primary_key=True, autoincrement=False)
    show_id = Column(Integer) # well, not needed..

    schedule_start = Column(DateTime) # can be null due to manual entries
    schedule_end = Column(DateTime) # can be null due to manual entries
    show_name = Column(String(256))
    show_hosts = Column(String(256))
    rtr_category = Column(String(256))
    comment = Column(String(512))
    languages = Column(String(256))
    type = Column(String(256))
    category = Column(String(256))
    topic = Column(String(256))
    musicfocus = Column(String(256))

    is_repetition = Column(Boolean())

    playlist_id = Column(Integer)
    timeslot_fallback_id = Column(Integer)
    show_fallback_id = Column(Integer)
    station_fallback_id = Column(Integer)

    def get_length(self):
        sec1 = int(datetime.datetime.strptime(self.start[0:16].replace(" ", "T"), "%Y-%m-%dT%H:%M").strftime("%s"))
        sec2 = int(datetime.datetime.strptime(self.end[0:16].replace(" ", "T"), "%Y-%m-%dT%H:%M").strftime("%s"))
        len = sec2 - sec1
        return len


# ------------------------------------------------------------------------------------------ #
class ScheduleEntry(db.Model, AuraDatabaseModel):
    """
    One schedule can have multiple entries
    """
    __tablename__ = 'schedule_entry'

    # primary and foreign keys
    playlist_id = Column(Integer, primary_key=True, nullable=False, autoincrement=False)
    entry_num = Column(Integer, primary_key=True, nullable=False, autoincrement=False)
    schedule_id = Column(Integer, ForeignKey("schedule.schedule_id"))

    entry_start = Column(DateTime) # can be null due to fallback playlists
    entry_end = Column(DateTime) # can be null due to fallback playlists
    source = Column(String(256))
    artist = Column(String(256))
    track = Column(String(256))
    albumname = Column(String(256))
    genre = Column(String(256))
    tracknum = Column(String(256))
    cdnum = Column(String(256))
    year = Column(Integer())
    volume = Column(Integer, default=100)
    is_fallback = Column(Boolean, default=False)
    entry_start_unix = 0
    entry_end_unix = 0
    programme_index = -1

    schedule = relationship("Schedule", foreign_keys=[schedule_id], lazy="joined")

    def __init__(self, **kwargs):
        super(ScheduleEntry, self).__init__(**kwargs)
        self.calc_unix_times()

    @orm.reconstructor
    def calc_unix_times(self):
        if self.entry_start is not None:
            self.entry_start_unix = time.mktime(self.entry_start.timetuple())
        if self.entry_end is not None:
            self.entry_end_unix = time.mktime(self.entry_end.timetuple())


    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def select_all():
        # when deleting all entries, and fetching new programmes, the entries are stored and commited in the code.
        # but sqlalchemy thinks somehow it is not commit and returns an empty set

        #print("WARNING: broadcasts.py This commit before SELECT is a BAND-AID & UGLY-HACK. Why the hell is there a transaction pending and not commited?")
        #db.session.commit()
        # fetching all entries
        all_entries = db.session.query(ScheduleEntry).filter(ScheduleEntry.is_fallback == false()).order_by(ScheduleEntry.entry_start).all()

        # BAND-AID debug output. The model and db session are different. crap
        #print("broadcasts.py SELECT ALL ScheduleEntry.q.session == db.session?")
        #print(ScheduleEntry.query.session == db.session)
        #print("broadcasts.py SELECT ALL ScheduleEntry.q.s.conn == db.s.conn?")
        #print(ScheduleEntry.query.session.connection() == db.session.connection())
        cnt = 0
        for entry in all_entries:
            entry.programme_index = cnt
            cnt = cnt + 1

        return all_entries

    @staticmethod
    def select_next_manual_entry_num():
        from sqlalchemy import func
        # damn BAND-AID
        #db.session.commit()

        #subqry = db.session.query(func.max(ScheduleEntry.entry_num)).filter(ScheduleEntry.schedule_id = 0)
        #qry = db.session.query(Data).filter(Data.user_id == user_id, Data.counter == subqry)

        max_manual_entry_num = db.session.query(func.max(ScheduleEntry.entry_num)).filter(ScheduleEntry.schedule_id == 0).first()
        print(max_manual_entry_num)

        if max_manual_entry_num[0] is None:
            return 0
        else:
            return int(max_manual_entry_num[0])+1

#        print("returning", res)

#        return res

    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def upcoming(datefrom=datetime.datetime.now()):
        # damn BAND-AID
        # db.session.commit()

        upcomingtracks = db.session.query(ScheduleEntry).filter(ScheduleEntry.start > datefrom).all()
        #upcomingtracks = ScheduleEntry.query.filter(ScheduleEntry.start > datefrom).all()
        return upcomingtracks

    # ------------------------------------------------------------------------------------------ #
    @staticmethod
    def select_one(playlist_id, entry_num):
        return db.session.query(ScheduleEntry).filter(ScheduleEntry.playlist_id == playlist_id, ScheduleEntry.entry_num == entry_num).first()

    # ------------------------------------------------------------------------------------------ #
    def __str__(self):
        return "ScheduleEntry starts @ " + ScheduleEntry.entry_start + " and ends @ " + ScheduleEntry.entry_end + " and plays " + ScheduleEntry.source


# ------------------------------------------------------------------------------------------ #
class TrackService(db.Model, AuraDatabaseModel):
    __tablename__ = 'trackservice'

    trackservice_id = Column(Integer, primary_key=True, autoincrement=True)
    schedule_id = Column(Integer, ForeignKey("schedule.schedule_id"))
    playlist_id = Column(Integer, nullable=False)
    entry_num = Column(Integer, nullable=False)

    source = Column(String(255), nullable=False)

    __table_args__ = (
        ForeignKeyConstraint(['playlist_id', 'entry_num'], ['schedule_entry.playlist_id', 'schedule_entry.entry_num']),
    )

    schedule = relationship("Schedule", foreign_keys=[schedule_id], lazy="joined")
    # trackservice_entry = relationship("ScheduleEntry", foreign_keys=[playlist_id, entry_num], lazy="joined")
    schedule_entry = relationship("ScheduleEntry", primaryjoin="and_(TrackService.playlist_id==ScheduleEntry.playlist_id, TrackService.entry_num==ScheduleEntry.entry_num)", lazy="joined")


# ------------------------------------------------------------------------------------------ #
# class TrackServiceSchedule(db.Model, AuraDatabaseModel):
#     """
#     Trackservice is tracking every schedule.
#     """
#     __tablename__ = 'trackservice_schedule'
#
#     # primary and foreign keys
#     ts_schedule_id = Column(Integer, primary_key=True, autoincrement=True)
#     schedule_id = Column(Integer, ForeignKey("schedule.schedule_id"))
#
#     schedule = relationship("Schedule", foreign_keys=[schedule_id], lazy="joined")
#
#     # ------------------------------------------------------------------------------------------ #
#     @staticmethod
#     def select_one(schedule_id):
#         # damn BAND-AID
#         # db.session.commit()
#
#         return db.session.query(ScheduleEntry).filter(TrackServiceSchedule.schedule_id == schedule_id).first()
#
# # ------------------------------------------------------------------------------------------ #
# class TrackServiceScheduleEntry(db.Model, AuraDatabaseModel):
#     """
#     And a schedule can have multiple entries
#     """
#     __tablename__ = 'trackservice_entry'
#
#     # primary and foreign keys. the foreign keys here can be null, because of fallback stuff
#     ts_entry_id = Column(Integer, primary_key=True, autoincrement=True)
#     ts_schedule_id = Column(Integer, ForeignKey("trackservice_schedule.ts_schedule_id"), nullable=True)
#     playlist_id = Column(Integer, nullable=True)
#     entry_num = Column(Integer, nullable=True)
#
#     fallback = Column(Boolean, default=False)
#     fallback_start = Column(DateTime, nullable=True, default=None)
#     source = Column(String(256), nullable=True, default=None)
#
#     # foreign key definitions
#     __table_args__ = (
#         ForeignKeyConstraint(['playlist_id', 'entry_num'], ['schedule_entry.playlist_id', 'schedule_entry.entry_num']),
#     )
#
#     trackservice_schedule = relationship("TrackServiceSchedule", foreign_keys=[ts_schedule_id], lazy="joined")
#     #trackservice_entry = relationship("ScheduleEntry", foreign_keys=[playlist_id, entry_num], lazy="joined")
#     trackservice_entry = relationship("ScheduleEntry", primaryjoin="and_(TrackServiceScheduleEntry.playlist_id==ScheduleEntry.playlist_id, TrackServiceScheduleEntry.entry_num==ScheduleEntry.entry_num)" , lazy="joined")
#
#     @staticmethod
#     def select_all():
#         return db.session.query(TrackServiceScheduleEntry).filter().all()

AuraDatabaseModel.recreate_db(True)
