import simplejson

class ExceptionLogger:
    # ------------------------------------------------------------------------------------------ #
    def __get_error__(self, job, errornumber, data):
        """
        Privat: Ermittelt Fehlermeldung, Job-Name (Klassenmethode) und Fehlercode für den Job aus error/controller_error.js
        @type errornumber:  string
        @param errornumber: Die interne Fehlernummer der aufrufenden Methode
        """
        ### weil es eine "bound method" ist, kommmt data als string an!???
        if data == None:
            data = {}
        if type(data) == type(str()):
            data = simplejson.loads(data)

        hasData = isinstance(data, (dict)) and len(data) > 0

        if job in self.errorData:
            errMsg = self.errorData[job][errornumber]
            errID = self.errorData[job]['id'] + str(errornumber)
            if hasData:
                for key in data.keys():
                    errMsg = errMsg.replace('::' + key + '::', str(data[key]))
            data['message'] = errMsg
            data['job'] = job
            data['code'] = errID
        return data

    # ------------------------------------------------------------------------------------------ #
    def success(self, job, data=None, errnum='00', value='', section='execjob'):
        """
        Erfolgsmeldung loggen
        @type errnum:    string
        @param errnum:   Errornummer der aufrufenden Funktion
        @type value:     string
        @param value:    Optionaler Wert
        @type section:   string
        @param section:  Gültigkeitsbereich
        """
        error = self.__get_error__(job, errnum, data)
        self.job_result = {'message': error['message'], 'code': error['code'], 'success': 'success',
                           'job': error['job'], 'value': value, 'section': section}
        self.redismessenger.send(error['message'], error['code'], 'success', error['job'], value, section)

    # ------------------------------------------------------------------------------------------ #
    def info(self, job, data=None, errnum='01', value='', section='execjob'):
        """
        Info loggen
        @type errnum:    string
        @param errnum:   Errornummer der aufrufenden Funktion
        @type value:     string
        @param value:    Optionaler Wert
        @type section:   string
        @param section:  Gültigkeitsbereich
        """
        error = self.__get_error__(job, errnum, data)
        self.job_result = {'message': error['message'], 'code': error['code'], 'success': 'info', 'job': error['job'],
                           'value': value, 'section': section}
        self.redismessenger.send(error['message'], error['code'], 'info', error['job'], value, section)

    # ------------------------------------------------------------------------------------------ #
    def warning(self, job, data=None, errnum='01', value='', section='execjob'):
        """
        Warnung loggen
        @type errnum:    string
        @param errnum:   Errornummer der aufrufenden Funktion
        @type value:     string
        @param value:    Optionaler Wert
        @type section:   string
        @param section:  Gültigkeitsbereich
        """
        error = self.__get_error__(job, errnum, data)
        self.job_result = {'message': error['message'], 'code': error['code'], 'success': 'warning',
                           'job': error['job'], 'value': value, 'section': section}
        self.redismessenger.send(error['message'], error['code'], 'warning', error['job'], value, section)

    # ------------------------------------------------------------------------------------------ #
    def error(self, job, data=None, errnum='01', value='', section='execjob'):
        """
        Error loggen
        @type errnum:    string
        @param errnum:   Errornummer der aufrufenden Funktion
        @type value:     string
        @param value:    Optionaler Wert
        @type section:   string
        @param section:  Gültigkeitsbereich
        """
        error = self.__get_error__(job, errnum, data)
        self.job_result = {'message': error['message'], 'code': error['code'], 'success': 'error', 'job': error['job'],
                           'value': value, 'section': section}
        self.redismessenger.send(error['message'], error['code'], 'error', error['job'], value, section)

    # ------------------------------------------------------------------------------------------ #
    def fatal(self, job, data=None, errnum='01', value='', section='execjob'):
        """
        Fatal error loggen
        @type errnum:    string
        @param errnum:   Errornummer der aufrufenden Funktion
        @type value:     string
        @param value:    Optionaler Wert
        @type section:   string
        @param section:  Gültigkeitsbereich
        """
        error = self.__get_error__(job, errnum, data)
        self.job_result = {'message': error['message'], 'code': error['code'], 'success': 'fatal', 'job': error['job'],
                           'value': value, 'section': section}
        self.redismessenger.send(error['message'], error['code'], 'fatal', error['job'], value, section)

    # ------------------------------------------------------------------------------------------ #
    def __check_result__(self, result):
        """
        Fehlerbehandlung
        @type     result: string
        @param    result: Ein Json-String
        """
        try:
            self.lq_error = simplejson.loads(result)
        except:
            return False

        try:
            if self.lq_error['success'] == 'success':
                return True
            else:
                return False
        except:
            return False